/**
 * @class Class that provides work with data like with objects.
 */
Models = {
};

/**
 * @class Describes genre of the media content.
 * @param {object} genre - genre object
 * @param {object} parent - parent element
 * @returns {Models.ContentGenre}
 */
Models.ContentGenre = function (genre, parent) {
	
	parent = parent || null;
	/**
	 * @field {number} 
	 * @description genre id
	 */
	this.id = genre.id;
	/**
	 * @field {string}
	 * @description name of the genre
	 */
	this.name = genre.name;
	/**
	 * @field 
	 * @description parent element
	 * @default null
	 */
	this.parent = parent;
	/**
	 * @field {boolean}
	 * @description flag whether has sub categories
	 */
	this.hasSubCategories = genre.childrenIDs.length > 0;
	/**
	 * @field {Array}
	 * @description sub categories of the genre
	 * @default []
	 */
	this.subCategories = [];
};

/**
 * @class Describes tree of the genres of the media content.
 * @param {Array} genres - array of the genre objects
 * @param {object} parent - parent element.
 * @returns {Array} children of the element.
 */
Models.ContentGenresTree = function (genres, parent) {
	parent = parent || null;
	
	var parentId = parent != null ? parseInt(parent.id) : 0;

	var children = [];
	for (var i = 0; i < genres.length; i++) {
		if (parseInt(genres[i].parentID) == parentId) {
			var newGenre = new Models.ContentGenre(genres.splice(i--, 1)[0], parent);
			newGenre.subCategories = Models.ContentGenresTree(genres, newGenre);
			children.push(newGenre);
		}
	}
	
	return children;
};

/**
 * @class Describes one content category.
 * @param id - category identifier
 * @param name - category name
 * @param hasSubCategories - flag whether has sub categories
 * @param parent - parent element
 * @param tag - category tag
 * @param displayOptions - options for the displaying
 * @returns {Models.Category}
 */
Models.Category = function (id, name, hasSubCategories, parent, tag, displayOptions) {
	/**
	 * @field
	 * @description category identifier
	 */
	this.id = id;
	/**
	 * @field 
	 * @description category name
	 */
	this.name = name;
	/**
	 * @field
	 * @description flag whether has sub categories
	 */
	this.hasSubCategories = hasSubCategories;
	/**
	 * @field
	 * @description array of the sub categories of the genre
	 */
	this.subCategories = hasSubCategories ? [] : null;
	/**
	 * @field
	 * @description parent element
	 */
	this.parent = parent;
	/**
	 * @field
	 * @description category tag
	 */
	this.tag = tag;
	/**
	 * @field
	 * @description options for the displaying
	 */
	this.displayOptions = displayOptions;
};

/**
 * @class Describes category of the channel.
 * @param channel - channel.
 * @returns {Models.ChannelCategory}
 */
Models.ChannelCategory = function (channel) {
	/**
	 * @field
	 * @description channels's category identifier
	 */
	this.id = parseInt(channel.id);
	/**
	 * @field
	 * @description channels's category name
	 */
	this.name = channel.name;
	/**
	 * @field
	 * @description flag whether has sub categories
	 * @default true
	 */
	this.hasSubCategories = true;
	/**
	 * @field
	 * @description array of the sub categories
	 * @default []
	 */
	this.subCategories = [];
	/**
	 * @field
	 * @description parent element
	 * @default null
	 */
	this.parent = null;
	/**
	 * @field
	 * @description flag whether element is channel
	 * @default true
	 */
	this.isChannel = true;
	/**
	 * @field
	 * @description options for the displaying
	 */
	this.advisory = channel.advisory;
	if (channel.arX && channel.arY) {
		/**
		 * @field
		 * @description channel element position
		 */
		this.ar = {
			x: channel.arX,
			y: channel.arY
		};
	}

	var self = this;
	/**
	 * @function 
	 * @description Provides update sub categories of the current object
	 */
	this.updateSubCategoriesParent = function () {
		for (var i = 0; i < self.subCategories.length; i++) {
			self.subCategories[i].parent = self;
		}
	};
};

/**
 * 
 * @param {Array} channels - collection of the channels
 * @returns {Array} collection of the channels categories.
 */
Models.CategoriesFromChannelsCollection = function (channels) {
	var outChannels = [];
	for (var i = 0; i < channels.length; i++) {
		outChannels[i] = new Models.ChannelCategory(channels[i]);
	}
	
	return outChannels;
};

/**
 * @class Describes request from ArchPlus.
 * @param {number} requestType - Defines search range for Video On Demand content (ArcPlus or VOD content type). ({@link JsonServices.OnDemandRequestType}).
 * @param {number} channelId - Requested items channel ID
 * @param {string} dateFrom - Date of an item. Date when an item was added to the video collection.
 * @param {string} dateTill - Upper limit of item date for range search
 * @param {number} genreId - Genre id.
 * @param {string} keyWord - A word to look for in item names (i.e. names of films, TV programs, series etc.)
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @returns {Models.ArchPlusRequest}
 */
Models.ArchPlusRequest = function (requestType, channelId, dateFrom, dateTill, genreId, keyWord, orderBy) {
	/**
	 * @field 
	 * @description Defines search range for Video On Demand content ({@link JsonServices.OnDemandRequestType})
	 */
	this.requestType = requestType;
	/**
	 * @field 
	 * @description Requested items channel ID
	 */
	this.channelId = channelId;
	/**
	 * @field 
	 * @description Date of an item. Date when an item was added to the video collection.
	 */
	this.dateFrom = dateFrom ? new Date(dateFrom.getTime()) : null;
	/**
	 * @field 
	 * @description Upper limit of item date for range search
	 */
	this.dateTill = dateTill ? new Date(dateTill.getTime()) : null;
	/**
	 * @field 
	 * @description Genre id.
	 */
	this.genreId = genreId;
	/**
	 * @field 
	 * @description A word to look for in item names (i.e. names of films, TV programs, series etc.)
	 */
	this.keyWord = keyWord;
	/**
	 * @field 
	 * @description Defines desired sort order of the search result set
	 */
	this.orderBy = orderBy;
	
	/**
	 * @function
	 * @description Return the number of milliseconds since 1970/01/01
	 * @param {object} date - date to convert.
	 * @returns {number} time in milliseconds
	 */
	this._getDateValue = function (date) {
		if (date) {
			return date.getTime();
		}
		
		return null;
	};
	
	/**
	 * @function 
	 * @description compares incoming request and request which saved in class
	 * @param {object} request
	 * @returns {boolean} result of compare.
	 */
	this.equals = function (request) {
		if (!request) {
			return false;
		}
		return  this.requestType === request.requestType &&
				this.channelId === request.channelId &&
				this._getDateValue(this.dateFrom) == this._getDateValue(request.dateFrom) &&
				this._getDateValue(this.dateTill) == this._getDateValue(request.dateTill) &&
				this.genreId === request.genreId &&
				this.keyWord === request.keyWord &&
				this.orderBy === request.orderBy;
	};
};

/**
 * @class Describes time zone.
 * @param uglyTimeZone - time zone information
 * @returns {Models.TimeZone}
 */
Models.TimeZone = function (uglyTimeZone) {
	/**
	 * @field
	 * @description time zone identifier
	 */
	this.id = uglyTimeZone.id;
	/**
	 * @field
	 * @description time zone name
	 */
	this.name = uglyTimeZone.name;
	/**
	 * @field
	 * @description when true the zone has shifted of live stream
	 */
	this.stream = uglyTimeZone.stream;
	/**
	 * @field
	 * @description Field "rus_name", to be replaced with a calculated field to represent name in user locale
	 */
	this.userName = uglyTimeZone.userName;
	/**
	 * @field
	 * @description Current UTC offset in minutes with DTS correction
	 */
	this.utcOffset = parseInt(uglyTimeZone.utcOffset);
	var fullName = "(UTC";
	if (this.utcOffset != 0) {
		fullName += " ";
		fullName += this.utcOffset > 0 ? "+" : "-";
		var utcOffset = Math.abs(this.utcOffset);
		fullName += Math.floor(utcOffset / 60) + ":";
		var minutes = utcOffset % 60;
		minutes = minutes < 10 ? "0" + minutes : minutes;
		fullName += minutes;
	}
	fullName += ") " + this.userName;
	/**
	 * @field
	 * @description full name of the time zone.
	 */
	this.fullName = fullName;
};

/**
 * @function
 * @description Convert incoming collection to the array of the TimeZone class objects.
 * @param uglyTimeZones - collection of the time zones to format. 
 * @returns {Array} collection of the time zones.
 */
Models.TimeZonesFromCollection = function (uglyTimeZones) {
	if (!uglyTimeZones) {
		return;
	}
	
	var timeZones = [];
	for (var i = 0; i < uglyTimeZones.length; i++) {
		timeZones.push(new Models.TimeZone(uglyTimeZones[i]));
	}
	
	return timeZones;
};