/**
 * @author Created by VasiliyK (Profit-Labs Ltd.)
 */
 
/**
 * @class Class describes Panel element (data element).
 * @param {string} id - element ID
 * @param {string} text - element text
 * @param {string} imgUrl - url of the image
 * @returns {PanelElement}
 */
PanelElement = function(id, text, imgUrl)
{
	/**
	 * Identifier of the element
	 */
    this.Id = id;
    /**
     * Text of the element
     */
    this.Text = text;
    /**
     * Url of the element image
     */
    this.imgUrl = imgUrl;
};

/**
 * @class [ENUM] Describes orientation (horizontal, vertical).
 */
var orientations = {
		/**
		 * [READONLY]
		 * @field
		 * @description horizontal orientation 
		 * @default 0
		 */
    horizontal : 0,
    /**
	 * [READONLY]
	 * @field
	 * @description vertical orientation 
	 * @default 1
	 */
    vertical : 1
};
 
/**
 * @class <p>Class describes Sliding panel <br> <i>(panel have absolute position, so it have to be placed in another element with absolute position)</i></p>
 * @param {number} initTop - initial top value.
 * @param {number} initLeft - initial left value.
 * @param {Array} elements - elements array (see - PanelElement constructor)
 * @param {string} idPrefix - service prefix, describes what type of content showing (SortBy, Category, etc.)
 * @param {function} selectedChangedCallback - callback, calls when selected element changes.
 * @param {number} elementSize - width of the element containing the context.
 * @param {number} orientation - element orientation (horiz., vertic.) ({@link orientations})
 * @param {boolean} alwaysSlide - type of the sliding (true - panel sliding to show next focus; false - focused item moving, panel slide when item out of visible zone.)
 * @param {number} visibleSize - size of the visible part of the panel.
 * @param {string} cssClassName - css class name of the element.
 * @param {number} delay - time (in ms) before loading data.
 * @param {boolean} hideElements - show or hide panel elements.
 * @param {number} visibleNum - count of elements for displaying.
 * @param {number} hideNum - count of not displayed elements.
 * @returns {SlidingPanel}
 */
SlidingPanel = function(initTop, initLeft, elements, idPrefix, selectedChangedCallback, 
                        elementSize, orientation, alwaysSlide, visibleSize, cssClassName, delay, hideElements, visibleNum, hideNum)
{
	/**
	 * Describes maximal length of the category name
     * @default 17
     */
	this.maxCategoryNameLength = 17;
	/**
	 * Count of elements for displaying.
     * @default 100
     */
    this.visibleNum = 100;
    /**
     * Count of not displayed elements.
     * @default 1
     */
    this.hideNum = hideNum || 1;
    /**
     * Time (in ms) before loading data.
     * @default 0
     */
    this.delay = delay || 0;
    /**
     * Describes success of data loading.
     * @default false
     */
    this.isDataLoaded = false;
    /**
     * Describes maximal width of the sliding panel.
     * @default 960
     */
    this.maxWidth = 960;
    /**
     * Describes initialization position (left offset).
     */
    this.initLeft = initLeft;
    /**
     * Describes initialization position (top offset).
     */
    this.initTop = initTop;
    /**
     * Describes orientation of the sliding.
     */
    this.orientation = orientation;
    /**
     * service prefix, describes what type of content showing (SortBy, Category, etc.)
     * @default "Element"
     */
    this.idPrefix = idPrefix || "Element";
    /**
     * Describes css class name of the element.
     * @default "slidingElement"
     */
    this.cssClassName = cssClassName || "slidingElement";
    elementSize = elementSize || 250;
    orientation = orientation || orientations.horizontal;
    if (typeof(alwaysSlide) == "undefined")
    {
        alwaysSlide = true;
    }
    
    if (!alwaysSlide && arguments.length < 9)
    {
        throw "When alwaysSlide == false, visibleSize cant't be undefined";
    }
    
    if (typeof(hideElements) == "undefined")
    {
    	/**
    	 * Show or hide panel elements.
    	 */
        this.hideElements = false;
    } 
    else
    {
        this.hideElements = true;
    } 
        
    if (this.hideElements && arguments.length < 13)
    {
        throw "When hideElements == true, visibleNum cant't be undefined";
    }
    
    if (this.hideElements)
    {
        this.visibleNum = visibleNum;
    }
    
    /**
    * @function 
    * @description Sets sliding panel top
    * @param {number} top - top offset in px.
    */
    this.setTop = function(top)
    {
        $(this.panel).css('top', top.toString()+"px");
    };
    
    /**
     * @function 
     * @description Sets sliding panel left
     * @param {number} left - left offset in px.
     */
    this.setLeft = function(left)
    {
        $(this.panel).css('left', left.toString()+"px");
    };
    
    this.panel = $("<div></div>");
    $(this.panel).css({
        'top' : initTop.toString()+"px",
        'left' : initLeft.toString()+"px",
        'position' : 'absolute'
    });
    
    /**
     * @function 
     * @description Provides loading data from the server.
     * @param {Array} elements - array of the panel elements.
     * @param {boolean} saveFocus - save focus on element or not.
     */
    this.loadData = function (elements, saveFocus)
    {
        this.isDataLoaded = false;
        $(this.panel).empty();
        this.elements = [];
        var tmpHtml = "";

        for (var i = 0; i < elements.length; i++)
        {
            var elementStyle = "";
            if (this.orientation == orientations.horizontal)
            {
                elementStyle = "left:"+(i*elementSize).toString()+"px;width:"+elementSize+"px;height:40px;";
            }
            else
            {
                elementStyle = "top:"+(i*elementSize).toString()+"px;width:280px;height:"+elementSize+"px;";
            }
            
            var tmpId = "div"+this.idPrefix+elements[i].Id;
            this.elements[i] = "#"+tmpId;
            var element= "";
            element += "<div id='"+tmpId+"' class='"+this.cssClassName;
            element += (this.orientation == orientations.horizontal) ? " inactive": "";
            
            var elementContent = elements[i].Text;
            element += (elementContent.indexOf(">") == (elementContent.length - 1)) ? " folder" : "";
            elementContent = elementContent.replace(">","");
			if (this.orientation == orientations.vertical) {
				elementContent = elementContent.length > this.maxCategoryNameLength ? elementContent.substring(0, this.maxCategoryNameLength) + "..." : elementContent;
			}
            
            element += "' style='"+elementStyle+"'><div class='bracketLeft' />";
            if (typeof(elements[i].imgUrl) != "undefined" && elements[i].imgUrl != "")
            {
                var imgId = "img"+this.idPrefix+elements[i].Id;
                var imageHtml = "<img title='"+elements[i].imgUrl+"' id='"+imgId+"' width='32' height='32' class='slidingPanelImage' />";
				elementContent = "<div class='channelLogo'>" + imageHtml + "</div><div style='display:inline'>" + elementContent + "</div>";
            }
            element += elementContent;
            element += (element.indexOf("folder") != - 1) && (this.orientation != orientations.horizontal) ? "<div class='bracketRight folderImg'></div></div>" : "<div class='bracketRight'></div></div>";
            tmpHtml += element;
        }
        $(this.panel).html(tmpHtml);
		
		if (!saveFocus || this.index < 0) {
			this.index = 0;
		}
		
        this.focusElement(this.index);
        this.blurPanel();
		
		this.slideToInit(true);
        this.isDataLoaded = true;
        tmpHtml = null;
    };
    
    /**
     * @function 
     * @description Provides unloading this class.
     */
    this.unload = function()
    {
        $(this.panel).remove();
        this.panel = null;
        this.elements.length = 0;
        this.elements = null;
        this.isDataLoaded = false;
        this.maxWidth = null;
        this.initLeft = null;
        this.initTop = null;
        this.orientation = null;
        this.idPrefix = null;
        this.cssClassName = null;
    };
    
    /**
     * @function 
     * @description Provides hide elements.
     */
    this.hideElems = function(){
        if (this.hideElements && this.index > 0)
        {
            var element = $(this.elements[this.index - 1], this.panel);
            element.hide();
        }
        
        if (this.hideElements)
        {
            for (var i = this.index - 1; i >= (this.index - this.hideNum) && i >= 0; i--)
            {
                var element = $(this.elements[i], this.panel);
                element.hide();
            }
            for (var i = this.index + this.visibleNum; i <= (this.index + this.visibleNum+  this.hideNum) && i < this.elements.length; i++)
            {
                var element = $(this.elements[i], this.panel);
                element.hide();
            }
            this.showVisibleNum();
        }
    };
    
    /**
     * @function 
     * @description Focuses element in panel with selected element
     * @param {number} index - index of the element to focus
     */
    this.focusElement = function(index)
    {
    	Logger.write(index, "SlidingPPPPanel", true);
        var element = $(this.elements[index], this.panel);
        
        if (this.orientation == orientations.horizontal)
        {      
				element.addClass("focused");
                
                var forEnd = (index + 3) > this.elements.length ? this.elements.length : index + 3;
                
                for (var i = index; i < forEnd; i++)
                {
                    var name = this.elements[i].replace("div","img");
                    
                    if (($(name).attr("src") == "") || (typeof($(name).attr("src")) == "undefined")) {
                        $(name).attr("src",$(name).attr("title"));
					}
                }
        }
        else
        {
			var classes = ""+ element.attr('className');
			element.css({'color':'#FFFFFF', 'z-index':'6'});
			element.css({"backgroundImage":"url(images/button-bgHover.png)"});
			if(classes.indexOf("folder") != -1)
			{
				$(this.elements[index] + " > .bracketRight").addClass("folderImgW");
				$(this.elements[index] + " > .bracketRight").removeClass("folderImg");
			}
        }   
        
        if (index != this.index)
        {
            var spacing = 0;
            
            if (alwaysSlide)
            {
                if (this.orientation == orientations.horizontal)
                {
                    spacing = this.initLeft - parseInt(element.css('left'));
                }
                else
                {
                    spacing = this.initTop - parseInt(element.css('top'));
                }
            }
            else
            {
                var relativeSpacing = 0;
                var space = 0;
                if (this.orientation == orientations.horizontal)
                {
                    relativeSpacing = parseInt(element.css('left')) + parseInt($(this.panel).css('left'));
                    space = parseInt(element.css('left'));
                    spacing = parseInt($(this.panel).css('left'));
                }
                else
                {
                    relativeSpacing = parseInt(element.css('top')) + parseInt($(this.panel).css('top'));
                    space = parseInt(element.css('top'));
                    spacing = parseInt($(this.panel).css('top'));
                }
                
                if (relativeSpacing + elementSize > visibleSize)
                {
                    spacing = visibleSize - space - elementSize;
                }
                else if (relativeSpacing < 0)
                {
                    spacing = -space;
                }
            }
            
            this.index = index;
            this.slide(spacing);
            if (this.isDataLoaded)
            {
                this.invokeSelectedChangedCallback();
            }
        }
    };
	
    /**
     * @function 
     * @description Blurs element in panel with selected element
     * @param {number} index - index of the element to blur
     */
    this.blurElement = function(index)
    {
        Logger.write("=====Bluring element: "+this.elements[index], "SlidingPanel");
        var element = $(this.elements[index], this.panel);
        if (this.orientation == orientations.horizontal)
        {
			element.removeClass("focused");

			var name = this.elements[index].replace("div","img");
			$(name).attr("src", "");
        }
        else
        {
			element.css({"backgroundImage":"none"});
			var classes = ""+ element.attr('className'); 
			element.css({'color':'#72C6F8', 'z-index':'5'});
			if(classes.indexOf("folder") != -1)
			{
				$(this.elements[index] + " > .bracketRight").addClass("folderImg");
				$(this.elements[index] + " > .bracketRight").removeClass("folderImgW");
			}
			element.css({"backgroundImage":"none"});
        }
    };
    
    /**
     * @function 
     * @description Occurs when user change selected item.
     */
    this.invokeSelectedChangedCallback = function()
    {
        var cb = function (self) {
			self.runCallback(self);
		};
		
        var self = this;
        
        if (typeof(this.selectedChangedTimeout) != "undefined")
        {
			clearTimeout(this.selectedChangedTimeout);
        }
        
		/* var e = new Error("message");*/
        (this.delay > 0) ? this.selectedChangedTimeout = setTimeout(function(){cb(self);}, this.delay) : cb(self);          
    };
	
    /**
     * @function 
     * @description Stopping panel's timer.
     */
	this.stopTimer = function () {
		if (this.selectedChangedTimeout) {
			clearTimeout(this.selectedChangedTimeout);
		}
	};
    
	/**
     * @function 
     * @description Start callback of the changing selection.
     * @param {object} self - sender (Sliding panel)
     */
    this.runCallback = function (self) {
        selectedChangedCallback(self.elements[self.index], self.index);
    };
    
    /**
     * @function 
     * @description Slides panel to new position.
     * @param {number} newPosition - new position of the panel (left or top, relative to orientation).
     * @param {boolean} withoutAnimation - sliding with or without animation.
     */
    this.slide = function (newPosition, withoutAnimation) {
		if (withoutAnimation) {
			if (this.orientation == orientations.horizontal) {
				this.setLeft(this.initLeft);
			} else {
				this.setTop(this.initTop);
			}
		} else {
			if (this.orientation == orientations.horizontal) {
				$(this.panel).animate({ 'left': newPosition.toString() }, 200);
			} else {
				$(this.panel).animate({ 'top':newPosition.toString() }, 200);
			}
		}
    };
	
    /**
     * @function 
     * @description Slides panel to initialization position.
     * @param {boolean} withoutAnimation - sliding with or without animation.
     */
	this.slideToInit = function (withoutAnimation) {
		if (this.orientation == orientations.horizontal) {
				this.slide(this.initLeft, withoutAnimation);
			} else {
				this.slide(this.initTop, withoutAnimation);
			}
	};
    
	/**
     * @function 
     * @description Hide the panel.
     */
    this.hide = function()
    {
        $(this.panel).css('visibility', 'hidden');
    };
    
    /**
     * @function 
     * @description Show the panel.
     */
    this.show = function()
    {
        $(this.panel).css('visibility', 'visible');
    };
   
    /**
     * @function 
     * @description Sets new element index (blurs old one and focuses new)
     * @param {number} newIndex - index of the new element
     */
    this.setIndex = function (newIndex) {
        /*if visible num setted*/
        if (this.hideElements && this.index > newIndex)
        {
            var element = $(this.elements[this.index + this.visibleNum], this.panel);
            Logger.write("hide elem="+this.index + this.visibleNum, "SlidingPanel");
            element.hide();
        }    
        /*if visible num setted*/        
        
        
        if (newIndex >= 0 && newIndex < this.elements.length)
        {
            this.blurElement(this.index);
            this.focusElement(newIndex);
        }
        
        /*if visible num setted*/
        this.hideElems();
          /*if visible num setted*/ 
    };
    
    /**
     * @function 
     * @description Show visible elements
     */
    this.showVisibleNum = function () {
        for (var i = this.index; i < (this.index + this.visibleNum); i++)
        {
            var elem = $(this.elements[i], this.panel);                
            elem.show();
        }
    };
    
    /**
     * @function 
     * @description Focuses panel
     */
    this.focusPanel = function()
    {
		this.isFocused = true;
        Logger.write("=====Focusing panel", "SlidingPanel");
        this.setIndex(this.index);
    };
    
    /**
     * @function 
     * @description Blurs panel
     */
    this.blurPanel = function()
    {
		this.isFocused = false;
        Logger.write("=====Bluring panel", "SlidingPanel");
        var element = $(this.elements[this.index], this.panel);
		if (this.orientation == orientations.horizontal)
		{
			element.removeClass("focused");
		}
		else
		{
			element.css({"backgroundImage":"url(images/black-0.3.png)"});
			element.css({"backgroundRepeat":"repeat"});
			element.css({'color':'#fff'});
			var classes = ""+ element.attr('className');
			if(classes.indexOf("folder") != -1)
			{
				$(this.elements[this.index] + " > .bracketRight").addClass("folderImg");
				$(this.elements[this.index] + " > .bracketRight").removeClass("folderImgW");
			}
		}
    };
    
    /**
     * @function 
     * @description Focuses next element
     */
    this.focusNext = function()
    {
        this.setIndex(this.index + 1);
    };
    
    /**
     * @function 
     * @description Focuses previous element
     */
    this.focusPrev = function()
    {
        this.setIndex(this.index - 1);
    };
    
    /**
     * @function 
     * @description Get focused element
     * @returns {object} focused element 
     */
    this.getFocused = function()
    {
        return $(this.elements[this.index], this.panel);
    };
    
    /**
     * @function 
     * @description Get focused element's id
     * @returns {number} focused element id.
     */
    this.getFocusedId = function()
    {
        return this.elements[this.index];
    };
	
    /**
     * @function 
     * @description Get elements count
     * @returns {number} elements count.
     */
	this.getTotal = function() {
		return this.elements.length;
	};
	
	/**
     * @function 
     * @description Get current position of the panel (left offset if horiz. orient.; top - vertic.).
     * @returns {number} position.
     */
	this.getCurrentPosition = function () {
		if (this.orientation == orientations.horizontal) {
			return parseInt($(this.panel).css("left"));
		} else {
			return parseInt($(this.panel).css("top"));
		}
	};
    
    this.loadData(elements);
    elements.length = 0;
    elements = null;
};